# -*- coding: utf-8 -*-

from django.conf.urls import url
from .views import PusherableExampleDetail


urlpatterns = [
    url(r'^(?P<pk>\d+)/$', PusherableExampleDetail.as_view(), name="example"),
]



# -*- coding: utf-8 -*-

import json

from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder
from django.forms.models import model_to_dict

from pusher import Pusher


class PusherMixin(object):
    pusher_include_model_fields = None
    pusher_exclude_model_fields = None
    
    def render_to_response(self, context, **response_kwargs):

        channel = u"{model}_{pk}".format(
            model=self.object._meta.model_name,
            pk=self.object.pk
        )
        
        data = self.__object_to_json_serializable(self.object)
        
        pusher = Pusher(app_id=settings.PUSHER_APP_ID,
                        key=settings.PUSHER_KEY,
                        secret=settings.PUSHER_SECRET)
        pusher.trigger(
            [channel, ],
            self.pusher_event_name,
            {
                'object': data,
                'user': self.request.user.username
            }
        )

        return super(PusherMixin, self).render_to_response(context, **response_kwargs)
        
    def __object_to_json_serializable(self, object):
        model_dict = model_to_dict(object, 
                                   fields=self.pusher_include_model_fields, exclude=self.pusher_exclude_model_fields)
        json_data = json.dumps(model_dict, cls=DjangoJSONEncoder)
        data = json.loads(json_data)
        return data


class PusherUpdateMixin(PusherMixin):
    pusher_event_name = u"update"

class PusherDetailMixin(PusherMixin):
    pusher_event_name = u"view"
    
class PusherDeleteMixin(PusherMixin):
    pusher_event_name = u"delete"



from django import template
from django.conf import settings

register = template.Library()

@register.simple_tag
def pusherable_script():
    return "<script src=\"//js.pusher.com/2.2/pusher.min.js\" type=\"text/javascript\"></script>"


@register.simple_tag
def pusherable_subscribe(event, instance):

    channel = u"{model}_{pk}".format(
        model=instance._meta.model_name,
        pk=instance.pk
    )

    return """
    <script type=\"text/javascript\">
    var pusher = new Pusher('{key}');
    var channel = pusher.subscribe('{channel}');
    channel.bind('{event}', function(data) {{
      pusherable_notify('{event}', data);
    }});
    </script>
    """.format(
        key=settings.PUSHER_KEY,
        channel=channel,
        event=event
    )



# -*- coding: utf-8 -*-

from django.views.generic.detail import DetailView
from pusherable.mixins import PusherDetailMixin
from .models import PusherableExample


class PusherableExampleDetail(PusherDetailMixin, DetailView):
    model = PusherableExample
    template_name = "example.html"
